@@ -15,24 +15,29 @@
 #ifndef QSODATADIALOG_H
 #define QSODATADIALOG_H
 
-#include "wx/wxprec.h"
+#include <wx/wxprec.h>
 
 #ifdef __BORLANDC__
 	#pragma hdrstop
 #endif
 
 #ifndef WX_PRECOMP
-	#include "wx/wx.h"
+	#include <wx/wx.h>
 #endif
 
-#include "tqsllib.h"
 #include <wx/bmpbuttn.h>
 #include <wx/textctrl.h>
 #include <wx/wxhtml.h>
 
 #include <vector>
+#include <map>
+#include <string>
+
+#include "tqsllib.h"
 
 using std::vector;
+using std::map;
+using std::string;
 
 /** Display and edit QSO data
   *
@@ -47,6 +52,7 @@ class QSORecord {
 		_band = wxT("160M");
 		tqsl_initDate(&_date, NULL);
 		tqsl_initTime(&_time, NULL);
+		_extraFields.clear();
 	}
 
 	wxString _call, _freq, _rxfreq;
@@ -54,6 +60,22 @@ class QSORecord {
 	wxString _propmode, _satellite;
 	tQSL_Date _date;
 	tQSL_Time _time;
+	map <string, string> _extraFields;
+	bool operator == (const QSORecord other) {
+		if (_call == other._call && _freq == other._freq &&
+		    _rxfreq == other._rxfreq && _mode == other._mode &&
+		    _band == other._band && _rxband == other._rxband &&
+		    _propmode == other._propmode &&
+		    _date.year == other._date.year &&
+		    _date.month == other._date.month &&
+		    _date.day == other._date.day &&
+		    _time.hour == other._time.hour &&
+		    _time.minute == other._time.minute &&
+		    _time.second == other._time.second) {
+			return true;
+		}
+		return false;
+	}
 };
 
 typedef vector<QSORecord> QSORecordList;
@@ -82,6 +104,7 @@ class QSODataDialog : public wxDialog  {
 	void OnRecNew(wxCommandEvent&);
 	void OnRecDelete(wxCommandEvent&);
 	void OnFieldChanged(wxCommandEvent&);
+	void OnClose(wxCloseEvent&);
 	void SetRecno(int recno);
 	void UpdateControls();
 
@@ -102,6 +125,7 @@ class QSODataDialog : public wxDialog  {
 	wxString _filename;
 	wxHtmlHelpController *_help;
 	int _newrec;
+	bool _something_changed;
 };
 
 #endif
