@@ -97,6 +97,7 @@ enum {		// Window IDs
 	ID_PREF_FILE_LOG_TAB,
 	ID_PREF_FILE_CERTPWD,
 	ID_PREF_FILE_OLDCRYPTO,
+	ID_PREF_FILE_QSLMGR,
 	ID_PREF_FILE_LOGVFY,
 	ID_PREF_IGNORE_SECONDS,
 	ID_PREF_IGNORE_CALLSIGN,
@@ -118,7 +119,7 @@ enum {		// Window IDs
 	ID_PREF_ONLINE_MESSAGERE,
 	ID_PREF_ONLINE_VERIFYCA,
 	ID_PREF_ONLINE_UPD_CONFIGURL,
-        ID_PREF_ONLINE_UPD_CONFIGFILE,
+	ID_PREF_ONLINE_UPD_CONFIGFILE,
 	ID_PREF_ONLINE_CERT_CHECK,
 	ID_PREF_PROXY_ENABLED,
 	ID_PREF_PROXY_HOST,
@@ -150,6 +151,7 @@ class FilePrefs : public PrefsPanel {
 #else
 	wxDirPickerCtrl *dirPick;
 #endif
+	wxChoice *primaryQSLMgr;
 	DECLARE_EVENT_TABLE()
 };
 
