@@ -14,9 +14,14 @@
 
 #include <cstdlib>
 #include <iostream>
+#include <string>
 #include "tqsllib.h"
 
-using namespace std;
+using std::string;
+using std::ios;
+using std::cerr;
+using std::cout;
+using std::endl;
 
 int
 cb(int, const char *msg, void *) {
