@@ -97,8 +97,8 @@
 #endif
 #include <stdio.h>
 #include <stdarg.h>
-#include <windef.h>
-#include <winbase.h>
+#define WIN32_LEAN_AND_MEAN
+#include <windows.h>
 #include <wchar.h>
 #include <string.h>
 #include <stdlib.h>
